<?php

/*
 * Project name: Flatboard
 * Project URL: http://flatboard.free.fr
 * Author: Frédéric Kaplon and contributors
 * All Flatboard code is released under the MIT license.
*/

# Security constant
define('FLATBOARD',    		    TRUE);
const DS = '/';
const BASEPATH = '.';
const PATH_ROOT = __DIR__ . DS;

include PATH_ROOT . DS . 'init.php';
require_once LIB_DIR . 'Utils.lib.php';

if (!defined('HTML_BASEPATH'))  define('HTML_BASEPATH', 	rtrim(Util::baseURL(), '\\/') );
if (!defined('HTML_THEME_DIR')) define('HTML_THEME_DIR',	HTML_BASEPATH . DS . 'theme' .DS. 'bootstrap' .DS);

require_once LIB_DIR . 'HTMLForm.lib.php';
# Jeton de sécurité
$token = CSRF::generate();
/*
 *---------------------------------------------------------------
 * LANGUAGE DETECTION
 *---------------------------------------------------------------
 */
$language = substr($_SERVER['HTTP_ACCEPT_LANGUAGE'], 0, 5);
$language = str_replace(',', '-', $language);
$language = explode('-', $language);
$language = $language[0]. '-' .strtoupper($language[1]);
# 
if (file_exists(LANG_DIR . $language. '.php')) require_once LANG_DIR . $language. '.php';
else require_once LANG_DIR . 'en-US.php';
/*
 *---------------------------------------------------------------
 * DEBUG / INSTALLATION
 *---------------------------------------------------------------
 */
// Check if Flatboard is not already installed 
if(flatDB::isValidEntry('config', 'config'))
	die('Flatboard is already installed!');

// Check PHP version
if(!version_compare(phpversion(), '5.3', '>=')) 
    die('Current PHP version '.phpversion().', you need > 5.3. (ERR_202)');

// Check PHP modules
Util::checkModules();

/*
 *---------------------------------------------------------------
 * ALL CHECKS AND WRITE CONFIG FILE
 *---------------------------------------------------------------
 */
if(!flatDB::isValidEntry('config', 'config') && HTMLForm::check('title') && HTMLForm::check('description', 5, 250) && HTMLForm::check('admin') && HTMLForm::check('mail') &&  HTMLForm::check('lang') && CSRF::check($token) )
{
	/*
	 *---------------------------------------------------------------
	 * DIRECTORY & PERMISSIONS
	 *---------------------------------------------------------------
	 */
	$chmod_error = array();
	// Directory
	$data_permissions = array(
		UPLOADS_DIR,
	    DATA_DIR,
	    DATA_DIR . 'topic',
	    DATA_DIR . 'reply',
	    DATA_DIR . 'forum',
	    DATA_DIR . 'plugin',
	    DATA_DIR . 'config',	
	    BAN_DIR,    
	);
	
	foreach ( $data_permissions as $dir ) {
		$permissions = 0777;
		# Create folders
		if (!file_exists($dir)) {
			mkdir($dir, $permissions, true);
		}
		# Check permissions		
	    if ( substr( sprintf( '%o', fileperms( $dir ) ), -4 ) != $permissions ) {
	        if( !chmod( $dir, $permissions ) ) {
	            $chmod_error[] = $dir;
	        }
	    }
	}
	# Check errors permissions
	if( !empty( $chmod_error ) ) {
	    $chmod_error_message = 'ERROR: Unable to change directory permissions for: ';	    
	    foreach( $chmod_error as $dir ) {
	        $chmod_error_message .= $dir . ', ';
	    }
	    die( $chmod_error_message );
	}
	# Create blocklist.txt
	if (!file_exists(BAN_FILE)) {
		$f = fopen(BAN_FILE, 'a+');
		fwrite($f, '');
		fclose($f);	
	}
	# Create index.php in data for no listing
	@file_put_contents(DATA_DIR. 'index.php', "<?php exit;?>", 0666);
	
    # création de la clé
    $key = password_hash($_POST['admin'], PASSWORD_DEFAULT);
	# sauvegarde la clé du mot de passe dans un fichier
	@file_put_contents(DATA_DIR. 'key.php', "<?php define('KEY', '$key'); ?>", 0666);
	if(file_exists(DATA_DIR.'key.php')) include(DATA_DIR.'key.php');	
			
	/*
	 *---------------------------------------------------------------
	 * WRITE CONFIG
	 *---------------------------------------------------------------
	 */
    $config['title']       		  = HTMLForm::clean($_POST['title']);
    $config['description'] 		  = HTMLForm::clean($_POST['description']);       
    $config['admin']       		  = hash_hmac('sha1', $_POST['admin'], KEY);
    $config['mail'] 	   		  = HTMLForm::clean($_POST['mail']);
    $config['editor']      		  = 'bbcode';
    $config['lang'] 	   		  = HTMLForm::clean($_POST['lang']);
    $bannerStyle		   		  = (defined('FLATBOARD_PRO'))?'#3e5b99':'#000';
    	 
	$config = array(
	    'title' 				  => $config['title'],
	    'description'	 		  => $config['description'],
	    'theme' 				  => 'bootstrap',
	    'style' 				  => $bannerStyle,
	    'lang' 					  => $config['lang'],
	    'date_format' 			  => 'l jS F o, H:i',
	    'mail' 					  => $config['mail'],	    
	    'footer_text'			  => 'Code licensed under MIT License',
	    'announcement' 			  => '<h5>' .$lang['installed_title']. '</h5><p>' .$lang['installed_msg']. '</p>',
	    'ItemByPage' 			  => 8,	
	    'nb_page_scroll_infinite' => 1,
	    'admin' 				  => $config['admin'],
	    # Extras
		'private' 				  => false,	
		'math_captcha' 			  => false,		    
	    'editor' 				  => $config['editor'],
	    'homepage' 				  => 'index.php',
	    'blog_new' 				  => '',
	    'post_summary' 			  => 55,
	    'salt' 					  => $token,
	    'worker' 				  =>  array()	               
        );        
    # sauvegarde de la configuration          
	flatDB::saveEntry('config', 'config', $config);
	# sauvegarde de l'ordre des forums
	flatDB::saveEntry('config', 'forumOrder', array());
	# sauvegarde du fichier pour la liste des tags (v.1.0.1)
	flatDB::saveEntry('config', 'tags', array());
	# démarrage de la session en tant qu'administrateur
	session_start();
	$_SESSION['role'] = 'admin';
	/*
	 *---------------------------------------------------------------
	 * INSTALL FINISH, LET'S GO!
	 *---------------------------------------------------------------
	 */
	Util::redirect('index.php' . DS . 'forum');	
}

?>
<!doctype html>
<html lang="<?php echo $language ?>">
<head>
	<meta charset="utf-8">
	<meta name="viewport" content="width=device-width, initial-scale=1.0">
	<title><?php echo $lang['flatBoard_installer'] ?></title>
	<link rel="stylesheet" href="<?php echo HTML_THEME_DIR ?>assets/css/bootstrap.css" />
	<style type="text/css">		
	body {
	  display: -ms-flexbox;
	  display: flex;
	  -ms-flex-align: center;
	  align-items: center;
	  padding-top: 7%;
	}
	</style>
</head>

<body class="<?php echo (defined('FLATBOARD_PRO'))?'bg-dark':'bg-light' ?>">
	
	<div class="container">
	  <div class="row justify-content-md-center">
		<div class="col-6 my-3 p-4 bg-white rounded shadow-lg">  	  
		    
			<header class="text-center">
				<img class="mb-4" src="<?php echo HTML_THEME_DIR ?>assets/img/<?php echo (defined('FLATBOARD_PRO'))?'flatboard-pro':'flatboard' ?>.png" alt="Flatboard">    
				<span class="text-uppercase badge badge-dark"><?php echo (defined('FLATBOARD_PRO'))?'pro':'standard' ?> v.<?php echo VERSION ?></span>
				<h1><?php echo $lang['flatBoard_installer'] ?></h1>
				<p><?php echo $lang['welcome_installer'] ?></p>
			</header>
		
			<section class="text-muted">
			<?php echo
				HTMLForm::form('install.php',
		            HTMLForm::text('title',isset($config)? $config['title'] : '', 'text', '', 'site_title').
		            HTMLForm::text('description',isset($config)? $config['description'] : '', 'text', '', 'site_slogan').
		            HTMLForm::text('mail', isset($config)? $config['mail'] : '', 'email', '', 'site_mail'). 
		            HTMLForm::text('admin', isset($config)? $config['admin'] : '', 'password', '', 'your_admin_psw').'
					<div class="row">
					    <div class="col">'.
					    	HTMLForm::select('lang', langpack(), isset($config)? $config['lang'] : $language). '
					    </div>
					    <div class="col">'.
							HTMLForm::select('editor', array('bbcode'=> $lang['bbcode'], 'markdown'=> $lang['markdown']), isset($config)? $config['editor'] : '', '', 'editor_desc',true). '
						</div>
					</div>'.                   
		            HTMLForm::simple_submit('install', 'btn btn-primary btn-lg btn-block')
		        );		
			?>
			</section>
	
		</div>		
	  </div>
	</div>
	
</body>
</html>